@@ -102,6 +102,8 @@ pub trait Vecf64 {
     fn cosine(self, _v: &[f64]) -> f64; 
     /// Vector subtraction
     fn vsub(self, v: &[f64]) -> Vec<f64>;
+    /// Vector negtion
+    fn negv(self) -> Vec<f64>;
     /// Vector addition
     fn vadd(self, v: &[f64]) -> Vec<f64>;
     /// Vector magnitude
@@ -142,10 +144,15 @@ pub trait Vecf64 {
     fn binsearch(self, v: f64) -> usize;
     /// Merges two ascending sorted vectors
     fn merge(self, v: &[f64]) -> Vec<f64>;
+    /// Merges two sort indices, returns simply concatenated Vec<f64> and new sort index into it
+    fn merge_immutable(self, idx1: &[usize], v2: &[f64], idx2: &[usize]) -> ( Vec<f64>,Vec<usize> );
+    /// merge indices of two already concatenated sorted vectors
+    fn merge_indices(self, idx1:&[usize], idx2:&[usize]) -> Vec<usize>;
     /// Sort vector in a standard way
     fn sortf(self) -> Vec<f64>;
     /// Sorted vector, is wrapper for mergesort below
     fn sortm(self, ascending:bool) -> Vec<f64>;
+
     /// Ranking with only n*log(n) complexity, using 'mergesort'
     fn mergerank(self) -> Vec<usize>;
     /// Immutable merge sort, makes a sort index
@@ -234,7 +241,7 @@ pub trait VecVecu8 {
     /// Weighted geometric median, sorted eccentricities magnitudes, cpdf of the weights
     fn gmedian(self, eps:f64) -> Vec<f64>; 
     /// The weighted geometric median
-    fn wgmedian(self, ws: &[u8],eps: f64) -> Vec<f64>; 
+    fn wgmedian(self, ws:&[u8], eps: f64) -> Vec<f64>; 
 }
 
 /// Methods applicable to vector of vectors of <f64>
